from StringIO import StringIO
import unittest2 as unittest
from libcloud.compute.base import NodeImage, Node, NodeSize
from libcloud.compute.types import Provider, NodeState
from libcloud.compute.drivers.openstack import OpenStackNetwork

import mock
from littlechef_rackspace.api import RackspaceApi, Regions
from littlechef_rackspace.lib import Host


class RackspaceApiTest(unittest.TestCase):
    def setUp(self):
        self.username = 'username'
        self.key = 'deadbeef'

        self.pending_node = Node(id='id', name='name', public_ips=[], private_ips=[],
                                 state=NodeState.PENDING, driver=None)
        self.active_node = Node(id='id', name='name', public_ips=[ '50.2.3.4'], private_ips=[],
                                state=NodeState.RUNNING, driver=None)


    def test_list_images_instantiates_driver_with_user_and_password(self):
        with mock.patch("littlechef_rackspace.api.get_driver") as get_driver:
            driver = get_driver.return_value

            api = self._get_api(Regions.DFW)
            api.list_images()

            driver.assert_any_call(self.username, self.key,
                                   ex_force_auth_url="https://identity.api.rackspacecloud.com/v2.0",
                                   ex_force_auth_version="2.0")

    def test_list_images_instantiates_dfw_driver(self):
        with mock.patch("littlechef_rackspace.api.get_driver") as get_driver:
            api = self._get_api(Regions.DFW)
            api.list_images()

        get_driver.assert_any_call(Provider.RACKSPACE_NOVA_DFW)

    def test_list_images_instantiates_ord_driver(self):
        with mock.patch("littlechef_rackspace.api.get_driver") as get_driver:
            api = self._get_api(Regions.ORD)
            api.list_images()

            get_driver.assert_any_call(Provider.RACKSPACE_NOVA_ORD)

    def test_list_images_instantiates_lon_driver(self):
        with mock.patch("littlechef_rackspace.api.get_driver") as get_driver:
            api = self._get_api(Regions.LON)
            api.list_images()

            get_driver.assert_any_call(Provider.RACKSPACE_NOVA_LON)

    def test_list_images_instantiates_syd_driver(self):
        with mock.patch("littlechef_rackspace.api.RackspaceNovaSydNodeDriver") as SydDriver:
            api = self._get_api(Regions.SYD)
            api.list_images()

            SydDriver.assert_any_call(self.username, self.key,
                                      ex_force_auth_url="https://identity.api.rackspacecloud.com/v2.0",
                                      ex_force_auth_version="2.0")

    def _get_api_with_mocked_conn(self, conn):
        api = self._get_api(Regions.ORD)
        api._get_conn = mock.Mock(return_value=conn)
        return api

    def test_list_images_returns_image_information(self):
        conn = mock.Mock()
        api = self._get_api_with_mocked_conn(conn)

        lc_image1 = NodeImage('abc-def', 'Image 1', None)
        lc_image2 = NodeImage('fge-hgi', 'Image 2', None)

        conn.list_images.return_value = [lc_image1, lc_image2]

        self.assertEquals([{
                               'id': lc_image1.id,
                               'name': lc_image1.name
                           }, {
                               'id': lc_image2.id,
                               'name': lc_image2.name
                           }], api.list_images())

    def test_list_flavors_returns_size_information(self):
        conn = mock.Mock()
        api = self._get_api_with_mocked_conn(conn)

        lc_size1 = NodeSize('1', '256 MB image', None, None, None, None, None)
        lc_size2 = NodeSize('2', '512 MB image', None, None, None, None, None)

        conn.list_sizes.return_value = [lc_size1, lc_size2]

        self.assertEquals([{
                               'id': lc_size1.id,
                               'name': lc_size1.name
                           }, {
                               'id': lc_size2.id,
                               'name': lc_size2.name
                           }], api.list_flavors())

    def test_list_networks_returns_network_information(self):
        conn = mock.Mock()
        api = self._get_api_with_mocked_conn(conn)

        network1 = OpenStackNetwork(id="abcdef", cidr="192.168.0.0/16",
                                    name="awesome network",
                                    driver=None)
        conn.ex_list_networks.return_value = [network1]

        self.assertEquals([{ 'id': network1.id, 'name': network1.name, 'cidr': network1.cidr }],
                          api.list_networks())

    def test_creates_node(self):
        conn = mock.Mock()
        api = self._get_api_with_mocked_conn(conn)

        image_id = "5cebb13a-f783-4f8c-8058-c4182c724ccd"
        flavor_id = "2"
        node_name = "new-node"
        public_key = "ssh-file deadbeef dave@isis"
        public_key_io = StringIO(public_key)
        conn.create_node.return_value = self.active_node

        api.create_node(node_name=node_name,
                        image=image_id,
                        flavor=flavor_id,
                        public_key_file=public_key_io)

        call_kwargs = conn.create_node.call_args_list[0][1]
        self.assertEquals(node_name, call_kwargs['name'])
        self.assertEquals(image_id, call_kwargs['image'].id)
        self.assertEquals(flavor_id, call_kwargs['size'].id)
        self.assertEquals({"/root/.ssh/authorized_keys": public_key},
                          call_kwargs['ex_files'])

    def test_creates_node_with_networks(self):
        conn = mock.Mock()
        api = self._get_api_with_mocked_conn(conn)
        network_id_list = ['id1', 'id2', 'id3']
        conn.create_node.return_value = self.active_node

        api.create_node(node_name="some name",
                        image="some image",
                        flavor="some flavor",
                        public_key_file=StringIO("some public key"),
                        networks=network_id_list)

        call_kwargs = conn.create_node.call_args_list[0][1]
        networks_kwarg = call_kwargs['networks']
        for network in networks_kwarg:
            self.assertIsInstance(network, OpenStackNetwork)

        self.assertEquals(network_id_list,
                          [network.id for network in networks_kwarg])

    def test_waits_for_node_to_become_active(self):
        conn = mock.Mock()
        api = self._get_api_with_mocked_conn(conn)

        conn.create_node.return_value = self.pending_node
        conn.ex_get_node_details.return_value = self.active_node

        with mock.patch('littlechef_rackspace.api.time') as time:
            api.create_node(node_name="some name",
                            image="5cebb13a-f783-4f8c-8058-c4182c724ccd",
                            flavor="2",
                            public_key_file=StringIO("some public key"))
            time.sleep.assert_any_call(5)

    def test_returns_host_information(self):
        conn = mock.Mock()
        api = self._get_api_with_mocked_conn(conn)

        public_ipv4_address = "50.51.52.53"
        self.active_node.public_ips = [
            "2222::2222:2",
            public_ipv4_address
        ]
        self.active_node.extra['password'] = 'password'
        conn.create_node.return_value = self.active_node

        result = api.create_node(node_name="some name",
                                 image="5cebb13a-f783-4f8c-8058-c4182c724ccd",
                                 flavor="2",
                                 public_key_file=StringIO("some public key"))

        self.assertEquals(result, Host(name="some name",
                                       ip_address=public_ipv4_address,
                                       password="password"))

    def test_outputs_progress_during_creation(self):
        conn = mock.Mock()
        api = self._get_api_with_mocked_conn(conn)

        progress = StringIO()

        conn.create_node.return_value = self.pending_node

        self.counter = 0
        def ex_get_node_details(id):
            if self.counter == 5:
                return self.active_node

            self.counter += 1
            return self.pending_node

        conn.ex_get_node_details.side_effect = ex_get_node_details
        password = 'abcDEFghiJKL'
        self.pending_node.extra['password'] = password

        with mock.patch('littlechef_rackspace.api.time') as time:
            node_name = "new node"
            image_id = "dontcare"
            flavor_id = "2"
            host = api.create_node(node_name=node_name,
                                   image=image_id,
                                   flavor=flavor_id,
                                   public_key_file=StringIO("some key"),
                                   progress=progress)

            self.assertEquals([
                "Creating node {0} (image: {1}, flavor: {2})...".format(node_name, image_id, flavor_id),
                "Created node {0} (id: {1}, password: {2})".format(node_name, self.pending_node.id, password),
                "Waiting for node to become active{0}".format("." * 6),
                "Node active! (host: {0})".format(host.ip_address)
            ], progress.getvalue().splitlines())

    def _get_api(self, region):
        return RackspaceApi(self.username, self.key, region)



import unittest2 as unittest
import mock
from littlechef_rackspace.lib import Host
from littlechef_rackspace.deploy import ChefDeployer

class ChefDeployerTest(unittest.TestCase):

    def setUp(self):
        self.host = Host(host_string="test.example.com",
                         ip_address="50.56.57.58")
        self.ohai_data = {
            'cloud': {
                'inet1_address': '1.2.3.4'
            },
            'network': {
                'eth0': 'eth 0 data',
                'eth1': 'eth 1 data'
            },
            'hostname': 'test',
            'keys': {
                'ssh': {
                    'public_keys': 'yep'
                }
            }
        }

    def _get_deployer(self, key_filename):
        deployer = ChefDeployer(key_filename=key_filename,)
        deployer._create_bootstrap_ssh_config = mock.Mock()

        return deployer

    @mock.patch('littlechef_rackspace.deploy.lc')
    @mock.patch('littlechef_rackspace.deploy.littlechef')
    def test_deploy_sets_fabric_settings(self, littlechef, lc):
        key_filename = "~/.ssh/bootstrap_rsa"
        deployer = self._get_deployer(key_filename=key_filename)
        deployer.deploy(self.host)

        self.assertEquals("root", lc.env.user)
        self.assertEquals(self.host.get_host_string(), lc.env.host_string)
        self.assertEquals(self.host.get_host_string(), lc.env.host)

    @mock.patch('littlechef_rackspace.deploy.lc')
    @mock.patch('littlechef_rackspace.deploy.littlechef')
    def test_deploy_calls_deploys_chef(self, littlechef, lc):
        deployer = self._get_deployer(key_filename="~/.ssh/id_rsa")

        deployer.deploy(self.host)

        lc.deploy_chef.assert_any_call(ask="no")

    @mock.patch('littlechef_rackspace.deploy.lc')
    @mock.patch('littlechef_rackspace.deploy.littlechef')
    def test_deploy_does_not_call_deploys_chef_if_options_is_set(self, littlechef, lc):
        deployer = self._get_deployer(key_filename="~/.ssh/id_rsa")

        deployer.deploy(self.host, use_opscode_chef=False)

        self.assertEquals(0, len(lc.deploy_chef.call_args_list))

    @mock.patch('littlechef_rackspace.deploy.lc')
    @mock.patch('littlechef_rackspace.deploy.littlechef')
    def test_deploy_with_no_runlist_calls_save_config(self, littlechef, lc):
        deployer = self._get_deployer(key_filename="~/.ssh/id_rsa")

        node_data = {
            'predefined': 'data'
        }
        littlechef.lib.get_node.return_value = node_data

        deployer.deploy(self.host)

        littlechef.lib.get_node.assert_any_call(self.host.host_string)
        node_data.update(self.ohai_data)
        littlechef.chef.save_config.assert_any_call(node_data, force=True)

    @mock.patch('littlechef_rackspace.deploy.lc')
    @mock.patch('littlechef_rackspace.deploy.littlechef')
    def test_deploy_without_hostname_gets_node_with_ip_address(self, littlechef, lc):
        deployer = self._get_deployer(key_filename="~/.ssh/id_rsa")

        node_data = {}
        self.host.host_string = None
        littlechef.lib.get_node.return_value = node_data

        deployer.deploy(self.host)

        littlechef.lib.get_node.assert_any_call(self.host.ip_address)

    @mock.patch('littlechef_rackspace.deploy.lc')
    @mock.patch('littlechef_rackspace.deploy.littlechef')
    def test_deploy_with_runlist_sets_node_runlist(self, littlechef, lc):
        deployer = self._get_deployer(key_filename="~/.ssh/id_rsa")

        predefined_data = {'run_list': []}
        littlechef.lib.get_node.return_value = predefined_data

        runlist = ['role[web]', 'recipe[apache2]']
        deployer.deploy(self.host, runlist=runlist)

        predefined_data.update(self.ohai_data)
        predefined_data['run_list'] = runlist

        littlechef.chef.save_config.assert_any_call(predefined_data, force=True)
        littlechef.lib.get_node.assert_any_call(self.host.host_string)

    @mock.patch('littlechef_rackspace.deploy.lc')
    @mock.patch('littlechef_rackspace.deploy.littlechef')
    def test_deploy_with_environment_sets_environment(self, littlechef, lc):
        deployer = self._get_deployer(key_filename="~/.ssh/id_rsa")
        self.host.environment = 'staging'

        expected_data = { 'chef_environment' : 'staging' }
        expected_data.update(self.ohai_data)
        littlechef.lib.get_node.return_value = expected_data

        deployer.deploy(self.host)

        littlechef.chef.save_config.assert_any_call(expected_data, force=True)

    @mock.patch('littlechef_rackspace.deploy.lc')
    @mock.patch('littlechef_rackspace.deploy.littlechef')
    def test_deploy_with_runlist_creates_temporary_ssh_config_file(self, littlechef, lc):
        deployer = self._get_deployer(key_filename="~/.ssh/id_rsa")

        deployer.deploy(self.host)

        expected_ssh_config = """User root
IdentityFile ~/.ssh/id_rsa
StrictHostKeyChecking no
Host test.example.com
HostName 50.56.57.58
"""

        deployer._create_bootstrap_ssh_config.assert_any_call(
            "./.bootstrap-config_{0}".format(self.host.get_host_string()),
            expected_ssh_config
        )

    @mock.patch('littlechef_rackspace.deploy.lc')
    @mock.patch('littlechef_rackspace.deploy.littlechef')
    def test_deploy_with_runlist_uses_temporary_ssh_config_values(self, littlechef, lc):
        deployer = self._get_deployer(key_filename="~/.ssh/id_rsa")

        deployer.deploy(self.host)

        self.assertTrue(lc.env.use_ssh_config)
        self.assertEquals(lc.env.ssh_config_path,
                          "./.bootstrap-config_{0}".format(self.host.get_host_string()))

    @mock.patch('littlechef_rackspace.deploy.lc')
    @mock.patch('littlechef_rackspace.deploy.littlechef')
    def test_deploy_with_runlist_runs_node_on_ip_address(self, littlechef, lc):
        deployer = self._get_deployer(key_filename="~/.ssh/id_rsa")

        deployer.deploy(self.host)

        lc.node.assert_any_call(self.host.host_string)

    @mock.patch('littlechef_rackspace.deploy.lc')
    @mock.patch('littlechef_rackspace.deploy.littlechef')
    def test_deploy_with_plugins_executes_plugins_on_node(self, littlechef, lc):
        deployer = self._get_deployer(key_filename="~/.ssh/id_rsa")

        plugin = mock.Mock()
        littlechef.lib.import_plugin.return_value = plugin
        node_data = {'predefined': [ 'values' ]}
        littlechef.lib.get_node.return_value = node_data

        deployer.deploy(self.host, plugins=['awesome_plugin'])

        littlechef.lib.import_plugin.assert_any_call('awesome_plugin')
        plugin.execute.assert_any_call(node_data)

    @mock.patch('littlechef_rackspace.deploy.lc')
    @mock.patch('littlechef_rackspace.deploy.littlechef')
    def test_deploy_with_plugin_prints_header(self, littlechef, lc):
        deployer = self._get_deployer(key_filename="~/.ssh/id_rsa")

        deployer.deploy(self.host, plugins=['plugin1'])

        littlechef.lib.print_header.assert_any_call(
            "Executing plugin '{0}' on {1}".format('plugin1', self.host.get_host_string())
        )

    @mock.patch('littlechef_rackspace.deploy.lc')
    @mock.patch('littlechef_rackspace.deploy.littlechef')
    def test_deploy_with_multiple_plugins(self, littlechef, lc):
        deployer = self._get_deployer(key_filename="~/.ssh/id_rsa")

        deployer.deploy(self.host, plugins=['plugin1', 'plugin2', 'plugin3'])

        littlechef.lib.import_plugin.assert_any_call('plugin1')
        littlechef.lib.import_plugin.assert_any_call('plugin2')
        littlechef.lib.import_plugin.assert_any_call('plugin3')

    @mock.patch('littlechef_rackspace.deploy.lc')
    @mock.patch('littlechef_rackspace.deploy.littlechef')
    def test_deploy_with_post_plugins(self, littlechef, lc):
        deployer = self._get_deployer(key_filename="~/.ssh/id_rsa")

        deployer.deploy(self.host, post_plugins=['postplugin'])

        littlechef.lib.import_plugin.assert_any_call('postplugin')


class Host(object):

    """
    Internal representation of a newly created/rebuilt server
    for Chef deployment, role addition, etc.
    """

    def __init__(self, name=None, host_string=None, ip_address=None, password=None, environment=None):
        self.name = name
        self.host_string = host_string
        self.ip_address = ip_address
        self.password = password
        self.environment = environment

    def get_host_string(self):
        if self.host_string:
            return self.host_string

        return self.ip_address

    def __repr__(self):
        return '<Host name={0}, host_string={1}, ip_address={2}, password={3}>'.format(
            self.name, self.host_string, self.ip_address, self.password
        )

    def __eq__(self, other):
        return (self.name == other.name and
                self.host_string == other.host_string and
                self.ip_address == other.ip_address and
                self.password == other.password and
                self.environment == other.environment)


from libcloud.compute.base import NodeImage, NodeSize
from libcloud.compute.drivers.openstack import OpenStackNetwork, OpenStack_1_1_NodeDriver
from libcloud.compute.drivers.rackspacenova import RackspaceNovaConnection
from libcloud.compute.providers import get_driver
from libcloud.compute.types import Provider, NodeState
import time
from lib import Host


# Hacks to support Sydney connections
class RackspaceNovaSydConnection(RackspaceNovaConnection):

    get_endpoint_args = {'service_type': 'compute',
                         'name': 'cloudServersOpenStack',
                         'region': 'SYD'}


class RackspaceNovaSydNodeDriver(OpenStack_1_1_NodeDriver):
    name = 'RackspaceNovaSyd'
    website = 'http://www.rackspace.com/'
    connectionCls = RackspaceNovaSydConnection
    api_name = 'rackspacenovaus'


class RackspaceApi(object):

    def __init__(self, username, key, region):
        self.username = username
        self.key = key
        self.region = region

    def _get_conn(self):
        if self.region is Regions.SYD:
            return RackspaceNovaSydNodeDriver(self.username, self.key,
                                              ex_force_auth_url="https://identity.api.rackspacecloud.com/v2.0",
                                              ex_force_auth_version="2.0")
        if self.region is Regions.DFW:
            provider = Provider.RACKSPACE_NOVA_DFW
        elif self.region is Regions.LON:
            provider = Provider.RACKSPACE_NOVA_LON
        else:
            provider = Provider.RACKSPACE_NOVA_ORD

        Driver = get_driver(provider)
        return Driver(self.username, self.key,
                      ex_force_auth_url="https://identity.api.rackspacecloud.com/v2.0",
                      ex_force_auth_version="2.0")

    def list_images(self):
        conn = self._get_conn()

        return [{ "id": image.id, "name": image.name}
                for image in conn.list_images()]

    def list_networks(self):
        conn = self._get_conn()

        return [{ "id": network.id, "name": network.name, "cidr": network.cidr }
                for network in conn.ex_list_networks()]

    def list_flavors(self):
        conn = self._get_conn()

        return [{ "id": size.id, "name": size.name}
                for size in conn.list_sizes()]

    def create_node(self, image, flavor, node_name, public_key_file, networks=None, progress=None):
        create_kwargs = {}
        if networks:
            fake_networks = [OpenStackNetwork(n, None, None, self) for n in networks]
            create_kwargs['networks'] = fake_networks

        conn = self._get_conn()
        fake_image = NodeImage(id=image, name=None, driver=conn)
        fake_flavor = NodeSize(id=flavor, name=None, ram=None, disk=None,
                               bandwidth=None, price=None, driver=conn)

        if progress:
            progress.write("Creating node {0} (image: {1}, flavor: {2})...\n".format(node_name, image, flavor))

        node = conn.create_node(name=node_name, image=fake_image,
                         size=fake_flavor, ex_files={
                             "/root/.ssh/authorized_keys": public_key_file.read()
                         },
                         **create_kwargs)
        password = node.extra.get("password")

        if progress:
            progress.write("Created node {0} (id: {1}, password: {2})\n".format(node_name, node.id, password))
            progress.write("Waiting for node to become active")

        while node.state != NodeState.RUNNING:
            time.sleep(5)

            if progress:
                progress.write(".")
            node = conn.ex_get_node_details(node.id)

        # Dumb hack to not select the ipv6 address
        public_ipv4_address = [ip for ip in node.public_ips if ":" not in ip][0]

        if progress:
            progress.write("\n")
            progress.write("Node active! (host: {0})\n".format(public_ipv4_address))
        return Host(name=node_name,
                    ip_address=public_ipv4_address,
                    password=password)

class Regions(object):

    NOT_FOUND = 0
    DFW = 1
    ORD = 2
    SYD = 3
    LON = 4



import unittest2 as unittest
import mock
from littlechef_rackspace.api import Regions, RackspaceApi
from littlechef_rackspace.commands import RackspaceCreate, RackspaceListImages
from littlechef_rackspace.deploy import ChefDeployer
from littlechef_rackspace.runner import Runner, MissingRequiredArguments, InvalidConfiguration, InvalidCommand, FailureMessages


class AbortException(Exception):
    """
    Abort mock must terminate execution of script
    """
    pass


class RunnerTest(unittest.TestCase):

    def setUp(self):
        self.api_class = mock.Mock(spec=RackspaceApi)
        self.rackspace_api = self.api_class.return_value
        self.deploy_class = mock.Mock(spec=ChefDeployer)
        self.chef_deployer = self.deploy_class.return_value
        self.create_class = mock.Mock(spec=RackspaceCreate)
        self.abort=mock.Mock()
        self.abort.side_effect = AbortException

        self.create_class.name = 'create'

        self.create_command = self.create_class.return_value
        self.list_images_class = mock.Mock(spec=RackspaceListImages)
        self.list_images_class.name = 'list-images'

        self.list_images_command = self.list_images_class.return_value

        # Dumb hacks using README.md as a public key because you can't mock out a file() call
        self.create_args = "create --flavor 2 --image 123 --node-name test-node --username username --key deadbeef --region dfw --public-key README.md".split(' ')

        list_images_command_string = "list-images --username username --key deadbeef --region REGION --public-key README.md"
        self.dfw_list_images_args = list_images_command_string.replace('REGION', 'dfw').split(' ')
        self.lon_list_images_args = list_images_command_string.replace('REGION', 'lon').split(' ')
        self.syd_list_images_args = list_images_command_string.replace('REGION', 'syd').split(' ')

    def test_must_specify_command(self):
        r = Runner(options={})
        with self.assertRaises(InvalidCommand):
            r.main([])

    def test_must_specify_existing_command(self):
        r = Runner(options={})
        with self.assertRaises(InvalidCommand):
            r.main(['bogus-command'])

    def test_list_images_fails_if_configuration_is_not_provided(self):
        with mock.patch.multiple("littlechef_rackspace.runner", RackspaceApi=self.api_class,
                                 ChefDeployer=self.deploy_class, RackspaceListImages=self.list_images_class,
                                 abort=self.abort):
            with self.assertRaises(AbortException):
                r = Runner(options={})
                r.main(["list-images"])
                self.abort.assert_any_call(FailureMessages.NEED_API_KEY)

    def test_list_images_with_dfw_region_instantiates_api(self):
        with mock.patch.multiple("littlechef_rackspace.runner", RackspaceApi=self.api_class,
                                 ChefDeployer=self.deploy_class, RackspaceListImages=self.list_images_class):
            r = Runner(options={})
            r.main(self.dfw_list_images_args)
            self.api_class.assert_any_call(username="username", key="deadbeef", region=Regions.DFW)

    def test_list_images_with_lon_region_instantiates_api(self):
        with mock.patch.multiple("littlechef_rackspace.runner", RackspaceApi=self.api_class,
                                 ChefDeployer=self.deploy_class, RackspaceListImages=self.list_images_class):
            r = Runner(options={})
            r.main(self.lon_list_images_args)
            self.api_class.assert_any_call(username="username", key="deadbeef", region=Regions.LON)

    def test_list_images_with_syd_region_instantiates_api(self):
        with mock.patch.multiple("littlechef_rackspace.runner", RackspaceApi=self.api_class,
                                 ChefDeployer=self.deploy_class, RackspaceListImages=self.list_images_class):
            r = Runner(options={})
            r.main(self.syd_list_images_args)
            self.api_class.assert_any_call(username="username", key="deadbeef", region=Regions.SYD)

    def test_uses_config_settings(self):
        with mock.patch.multiple("littlechef_rackspace.runner", RackspaceApi=self.api_class,
                                 ChefDeployer=self.deploy_class, RackspaceListImages=self.list_images_class):
            r = Runner(options={
                'username': 'username',
                'key': 'deadbeef',
                'region': 'ord'
            })
            # another dumb hack
            r.main(['list-images', '--public-key', 'README.md'])
            self.api_class.assert_any_call(username="username", key="deadbeef", region=Regions.ORD)

    def test_create_fails_if_configuration_is_not_provided(self):
        r = Runner(options={})
        with mock.patch.multiple('littlechef_rackspace.runner', abort=self.abort):
            with self.assertRaises(AbortException):
                r.main(["create"])
                self.abort.assert_any_call(FailureMessages.NEED_API_KEY)


    def test_create_fails_if_required_arguments_are_not_provided(self):
        with mock.patch.multiple('littlechef_rackspace.runner', abort=self.abort):
            with self.assertRaises(AbortException):
                self.create_command.validate_args.return_value = False
                r = Runner(options={})
                r.main("create --username username --key deadbeef --region dfw".split(" "))
                self.abort.assert_any_call(FailureMessages.MISSING_REQUIRED_ARGUMENTS)

    def test_create_instantiates_api_and_deploy_with_default_private_key(self):
        with mock.patch.multiple("littlechef_rackspace.runner", RackspaceApi=self.api_class,
                                 ChefDeployer=self.deploy_class, RackspaceCreate=self.create_class):
            r = Runner(options={})
            r.main(self.create_args)

            self.api_class.assert_any_call(username="username", key="deadbeef", region=Regions.DFW)
            self.deploy_class.assert_any_call(key_filename="~/.ssh/id_rsa")

    def test_create_creates_node_with_specified_public_key(self):
        with mock.patch.multiple("littlechef_rackspace.runner", RackspaceApi=self.api_class,
                                 ChefDeployer=self.deploy_class, RackspaceCreate=self.create_class):
            r = Runner(options={})
            r.main(self.create_args)

            self.create_class.assert_any_call(rackspace_api=self.rackspace_api,
                                              chef_deployer=self.chef_deployer)

            call_args = self.create_command.execute.call_args_list[0][1]

            self.assertEquals("123", call_args["image"])
            self.assertEquals("2", call_args["flavor"])
            self.assertEquals("test-node", call_args["node_name"])
            self.assertEquals('README.md', call_args['public_key_file'].name)

    def test_create_with_runlist_parses_runlist_into_array(self):
        with mock.patch.multiple("littlechef_rackspace.runner", RackspaceApi=self.api_class,
                                 ChefDeployer=self.deploy_class, RackspaceCreate=self.create_class):
            r = Runner(options={})
            runlist = 'role[test],recipe[web],recipe[apache2]'
            r.main(self.create_args + [ '--runlist', runlist ])

            call_args = self.create_command.execute.call_args_list[0][1]
            self.assertEquals(runlist.split(','), call_args["runlist"])

    def test_create_with_plugins_parses_plugins_into_array(self):
        with mock.patch.multiple("littlechef_rackspace.runner", RackspaceApi=self.api_class,
                                 ChefDeployer=self.deploy_class, RackspaceCreate=self.create_class):
            r = Runner(options={})
            plugins = 'plugin1,plugin2,plugin3'
            r.main(self.create_args + [ '--plugins', plugins ])

            call_args = self.create_command.execute.call_args_list[0][1]
            self.assertEquals(plugins.split(','), call_args["plugins"])

    def test_create_with_postplugins_parses_postplugins_into_array(self):
        with mock.patch.multiple("littlechef_rackspace.runner", RackspaceApi=self.api_class,
                                 ChefDeployer=self.deploy_class, RackspaceCreate=self.create_class):
            r = Runner(options={})
            post_plugins = 'plugin1,plugin2,plugin3'
            r.main(self.create_args + [ '--post-plugins', post_plugins ])

            call_args = self.create_command.execute.call_args_list[0][1]
            self.assertEquals(post_plugins.split(','), call_args["post_plugins"])

    def test_create_with_skip_opscode_chef_to_false(self):
        with mock.patch.multiple("littlechef_rackspace.runner", RackspaceApi=self.api_class,
                                 ChefDeployer=self.deploy_class, RackspaceCreate=self.create_class):
            r = Runner(options={})
            r.main(self.create_args + [ '--skip-opscode-chef'])

            call_args = self.create_command.execute.call_args_list[0][1]
            self.assertEquals(False, call_args["use_opscode_chef"])

    def test_create_without_skip_opscode_chef(self):
        with mock.patch.multiple("littlechef_rackspace.runner", RackspaceApi=self.api_class,
                                 ChefDeployer=self.deploy_class, RackspaceCreate=self.create_class):
            r = Runner(options={})
            r.main(self.create_args)

            call_args = self.create_command.execute.call_args_list[0][1]
            self.assertEquals(None, call_args.get("use_opscode_chef"))

    def test_create_with_use_opscode_chef_to_false(self):
        with mock.patch.multiple("littlechef_rackspace.runner", RackspaceApi=self.api_class,
                                 ChefDeployer=self.deploy_class, RackspaceCreate=self.create_class):
            r = Runner(options={})
            r.main(self.create_args + [ '--use-opscode-chef', '0'])

            call_args = self.create_command.execute.call_args_list[0][1]
            self.assertEquals(False, call_args["use_opscode_chef"])

    def test_create_with_use_opscode_chef_to_true(self):
        with mock.patch.multiple("littlechef_rackspace.runner", RackspaceApi=self.api_class,
                                 ChefDeployer=self.deploy_class, RackspaceCreate=self.create_class):
            r = Runner(options={})
            r.main(self.create_args + [ '--use-opscode-chef', '1'])

            call_args = self.create_command.execute.call_args_list[0][1]
            self.assertEquals(True, call_args.get("use_opscode_chef"))

    def test_create_with_use_opscode_chef_not_specified(self):
        with mock.patch.multiple("littlechef_rackspace.runner", RackspaceApi=self.api_class,
                                 ChefDeployer=self.deploy_class, RackspaceCreate=self.create_class):
            r = Runner(options={})
            r.main(self.create_args)

            call_args = self.create_command.execute.call_args_list[0][1]
            self.assertFalse("use_opscode_chef" in call_args)

    def test_create_with_networks_without_publicnet_raises_exception(self):
        with mock.patch.multiple("littlechef_rackspace.runner", RackspaceApi=self.api_class,
                                 ChefDeployer=self.deploy_class, RackspaceCreate=self.create_class):
            r = Runner(options={})
            with self.assertRaises(InvalidConfiguration) as cm:
                r.main(self.create_args + [ '--networks', 'abcdefg'])

            self.assertEquals('Must specify PublicNet in networks list (id=00000000-0000-0000-0000-000000000000)',
                              cm.exception.message)

    def test_create_with_networks_passes_networks(self):
        with mock.patch.multiple("littlechef_rackspace.runner", RackspaceApi=self.api_class,
                                 ChefDeployer=self.deploy_class, RackspaceCreate=self.create_class):
            r = Runner(options={})
            public_net_id = '00000000-0000-0000-0000-000000000000'
            custom_net_id= '45e8b288-3a98-4092-a3e8-37e2a540d004'
            r.main(self.create_args + [ '--networks', '{0},{1}'.format(public_net_id, custom_net_id)])

            call_args = self.create_command.execute.call_args_list[0][1]
            self.assertEquals([public_net_id, custom_net_id], call_args.get('networks'))



import ConfigParser
import os
from optparse import OptionParser
from fabric.utils import abort
import littlechef
from api import RackspaceApi, Regions
from deploy import ChefDeployer
from commands import RackspaceCreate, RackspaceListImages, RackspaceListFlavors, RackspaceListNetworks


def get_command_classes():
    return [RackspaceCreate,
            RackspaceListImages,
            RackspaceListFlavors,
            RackspaceListNetworks]


class FailureMessages:

    NEED_API_KEY = ('Must specify username, API key, and region on command line '
                    'or in [rackspace] configuration section of config.cfg')

    MISSING_REQUIRED_ARGUMENTS = "Missing required arguments"

    MUST_SPECIFY_PUBLICNET = 'Must specify PublicNet in networks list (id=00000000-0000-0000-0000-000000000000)'


class RackspaceOptionParser(OptionParser):
    def print_help(self, file=None):
        OptionParser.print_help(self, file)
        print "\nAvailable commands:\n"
        for command_class in get_command_classes():
            print "   {0}\t{1}".format(command_class.name,
                                       command_class.description)
        print ""


parser = RackspaceOptionParser()
parser.add_option("-I", "--image", dest="image",
                  help="Image ID")
parser.add_option("-f", "--flavor", dest="flavor",
                  help="Flavor ID")
parser.add_option("-A", "--username", dest="username",
                  help="Rackspace Username")
parser.add_option("-N", "--node-name", dest="node_name",
                  help="Node name")
parser.add_option("-K", "--key", dest="key",
                  help="Rackspace API Key")
parser.add_option("-R", "--region", dest="region", default="",
                  help="Region for provisioning (required for OpenStack)")
parser.add_option("-a", "--public-key", dest="public_key",
                  help="Public Key File for Bootstrapping")
parser.add_option("-i", "--private-key", dest="private_key",
                  help="Private Key File for Bootstrapping")
parser.add_option("-r", "--runlist", dest="runlist",
                  help="Node runlist delimited by commas, e.g. 'role[web],recipe[db]'")
parser.add_option("-e", "--env", dest="environment",
                  help="Environment for newly created node",
                  default=None)
parser.add_option("-H", "--hostname", dest="hostname",
                  help="Hostname for newly created node (DNS will not be set up -- you must do this manually)",
                  default=None)
parser.add_option("-p", "--plugins", dest="plugins",
                  help="Plugins to execute after chef bootstrapping but before chef run e.g, 'save_cloud,save_hosts'",
                  default=None)
parser.add_option("-P", "--post-plugins", dest="post-plugins",
                  help="Plugins to execute after chef run. e.g, 'mark_node_as_provisioned'",
                  default=None)
parser.add_option("--skip-opscode-chef", action="store_false", dest="use-opscode-chef")
parser.add_option("--use-opscode-chef", type="int", dest="use-opscode-chef",
                  help="Integer argument with whether or not to use the OpsCode Chef repositorities (installed with 'fix deploy_chef')",
                  default=None)
parser.add_option("-n", "--networks", dest="networks",
                  help="Comma separated list of network ids to create node with (PublicNet is required)",
                  default=None)

class Runner(object):
    def _read_littlechef_config(self):
        try:
            config = ConfigParser.SafeConfigParser()
            success = config.read(littlechef.CONFIGFILE)
            if success:
                return dict(config.items('rackspace'))
            else:
                abort("Could not read littlechef configuration file!  "
                      "Make sure you are running in a kitchen (fix new_kitchen).")
        except ConfigParser.ParsingError:
            pass
        except ConfigParser.NoSectionError as e:
            pass

        return None

    def __init__(self, options=None):
        self.command_classes = get_command_classes()
        if options is None:
            options = self._read_littlechef_config()

        self.options = options or {}

        secrets_file = self.options.get('secrets-file')
        if secrets_file:
            try:
                del self.options['secrets-file']
                secrets_file = os.path.expanduser(secrets_file)
                # yes, look at another config file
                secrets_config = ConfigParser.SafeConfigParser()
                _ = secrets_config.read(secrets_file)
                self.options.update(dict(secrets_config.items(ConfigParser.DEFAULTSECT)))
            except ConfigParser.ParsingError:
                pass

    def get_api(self):
        username = self.options.get('username')
        key = self.options.get('key')
        region = self.options.get('region', '')
        if region.lower() == 'dfw':
            region = Regions.DFW
        elif region.lower() == 'ord':
            region = Regions.ORD
        elif region.lower() == 'syd':
            region = Regions.SYD
        elif region.lower() == 'lon':
            region = Regions.LON
        else:
            region = Regions.NOT_FOUND

        if not username or not key or not region:
            abort(FailureMessages.NEED_API_KEY)

        return RackspaceApi(username=username, key=key, region=region)

    def get_deploy(self):
        key_filename = self.options.get("private_key", "~/.ssh/id_rsa")
        return ChefDeployer(key_filename=key_filename)

    def main(self, cmd_args):
        (options, args) = parser.parse_args(cmd_args)

        if not args:
            raise InvalidCommand

        user_command = args[0]
        matched_commands = filter(lambda command_class: command_class.name == user_command,
                                  self.command_classes)

        if not matched_commands:
            raise InvalidCommand

        for k, v in vars(options).items():
            if v is not None and v != '':
                self.options[k] = v

        command_class = matched_commands[0]
        command_kwargs = {'rackspace_api': None,
                          'chef_deployer': None}

        if command_class.requires_api:
            command_kwargs['rackspace_api'] = self.get_api()
        if command_class.requires_deploy:
            command_kwargs['chef_deployer'] = self.get_deploy()

        command = command_class(**command_kwargs)

        args = self.options

        if not command.validate_args(**args):
            abort(FailureMessages.MISSING_REQUIRED_ARGUMENTS)
            return

        public_key = args.get('public_key', "~/.ssh/id_rsa.pub")
        args['public_key_file'] = file(os.path.expanduser(public_key))

        if args.get('runlist'):
            args['runlist'] = args['runlist'].split(',')

        if args.get('plugins'):
            args['plugins'] = args['plugins'].split(',')

        if args.get('post-plugins'):
            args['post_plugins'] = args['post-plugins'].split(',')

        if 'use-opscode-chef' in args:
            args['use_opscode_chef'] = bool(args['use-opscode-chef'])

        if args.get('networks'):
            networks = args.get('networks').split(',')
            if '00000000-0000-0000-0000-000000000000' not in networks:
                raise InvalidConfiguration(FailureMessages.MUST_SPECIFY_PUBLICNET)

            args['networks'] = networks

        command.execute(**args)

class MissingRequiredArguments(Exception):
    pass


class InvalidConfiguration(Exception):
    pass


class InvalidCommand(Exception):
    pass



try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(
    name="littlechef-rackspace",
    version=__import__('littlechef_rackspace').__version__,
    description="Scripts for bootstrapping Rackspace Cloud Servers with Littlechef",
    author="Dave King",
    author_email="tildedave@gmail.com",
    url="http://github.com/tildedave/littlechef-rackspace",
    download_url="http://github.com/tildedave/littlechef-rackspace/archives/master",
    keywords=["chef", "rackspace", "openstack", "devops", "operations"],
    install_requires=['littlechef>=1.4.1', 'apache-libcloud==0.12.4'],
    packages=['littlechef_rackspace'],
    scripts=['fix-rackspace'],
    test_suite='nose.collector',
    classifiers=[
        "Programming Language :: Python",
        "Environment :: Console",
        "Intended Audience :: Developers",
        ],
    long_description="""\
Deploy Chef to Rackspace Cloud Servers without a chef server.  Replaces
knife-rackspace for users of littlechef.

.. _Chef: http://wiki.opscode.com/display/chef/Home
.. _LittleChef: https://github.com/tobami/littlechef
.. _Rackspace Cloud Servers: http://www.rackspace.com/cloud/servers/
"""
)



from StringIO import StringIO
import unittest2 as unittest
import mock
import sys
from littlechef_rackspace.api import RackspaceApi
from littlechef_rackspace.commands import RackspaceCreate, RackspaceListImages, RackspaceListFlavors, RackspaceListNetworks
from littlechef_rackspace.deploy import ChefDeployer
from littlechef_rackspace.lib import Host

class RackspaceCreateTest(unittest.TestCase):

    def setUp(self):
        self.api = mock.Mock(spec=RackspaceApi)
        self.deployer = mock.Mock(spec=ChefDeployer)
        self.command = RackspaceCreate(rackspace_api=self.api,
                                       chef_deployer=self.deployer)
        self.api.create_node.return_value = Host()

    def test_creates_host_in_api(self):
        node_name = "test"
        image = "imageId"
        flavor = "flavorId"
        public_key_file = StringIO("~/.ssh/id_rsa.pub")

        self.command.execute(node_name=node_name, image=image,
                             flavor=flavor, public_key_file=public_key_file)

        self.api.create_node.assert_any_call(node_name=node_name, image=image,
                                             flavor=flavor, public_key_file=public_key_file,
                                             networks=None,
                                             progress=sys.stderr)

    def test_deploys_to_host_with_kwargs(self):
        kwargs = {
            'runlist': ['role[web]', 'recipe[test'],
            'plugins': 'bootstrap',
            'post_plugins': 'all_done'
        }
        self.command.execute(node_name="something", image="imageId",
                             flavor="fileId", public_key_file=StringIO("whatever"),
                             **kwargs)

        expected_args = {
            'host': Host()
        }
        expected_args.update(kwargs)

        self.deployer.deploy.assert_any_call(**expected_args)

    def test_deploys_to_host_with_hostname(self):
        self.command.execute(node_name="something", image="imageId",
                             flavor="fileId", public_key_file=StringIO("whatever"),
                             hostname="test.example.com")

        self.deployer.deploy.assert_any_call(host=Host(host_string="test.example.com"))

    def test_deploys_to_host_with_environment(self):
        self.command.execute(node_name="something", image="imageId",
                             flavor="fileId", public_key_file=StringIO("whatever"),
                             environment='staging')

        expected_host = Host()
        expected_host.environment = 'staging'

        self.deployer.deploy.assert_any_call(host=expected_host)

class RackspaceListImagesTest(unittest.TestCase):

    def setUp(self):
        self.api = mock.Mock(spec=RackspaceApi)
        self.command = RackspaceListImages(rackspace_api=self.api)

    def test_outputs_images(self):
        progress = StringIO()
        image1 = {'id': '1', 'name': 'Ubuntu 12.04 LTS'}
        image2 = {'id': '2', 'name': 'Ubuntu 10.04 LTS'}
        self.api.list_images.return_value = [ image1, image2 ]

        self.command.execute(progress=progress)

        self.assertEquals([
                              '{0}{1}'.format(image1['id'].ljust(38 + 5), image1['name']),
                              '{0}{1}'.format(image2['id'].ljust(38 + 5), image2['name'])
                          ], progress.getvalue().splitlines())

class RackspaceListFlavorsTest(unittest.TestCase):

    def setUp(self):
        self.api = mock.Mock(spec=RackspaceApi)
        self.command = RackspaceListFlavors(rackspace_api=self.api)

    def test_outputs_flavors(self):
        progress = StringIO()
        flavor1 = {'id': '1', 'name': '256 MB'}
        flavor2 = {'id': '2', 'name': '512 MB'}
        self.api.list_flavors.return_value = [ flavor1, flavor2 ]

        self.command.execute(progress=progress)

        self.assertEquals([
                              '{0}{1}'.format(flavor1['id'].ljust(10), flavor1['name']),
                              '{0}{1}'.format(flavor2['id'].ljust(10), flavor2['name'])
                          ], progress.getvalue().splitlines())


class RackspaceListNetworksTest(unittest.TestCase):

    def setUp(self):
        self.api = mock.Mock(spec=RackspaceApi)
        self.command = RackspaceListNetworks(rackspace_api=self.api)

    def test_outputs_networks(self):
        progress = StringIO()
        network1 = { 'id': '0', 'name': 'PublicNet', 'cidr': None }
        network2 = { 'id': '1', 'name': 'My Test Network', 'cidr': '192.168.0.0/20' }
        network3 = { 'id': '2', 'name': 'ServiceNet', 'cidr': '10.0.0.0/20' }
        self.api.list_networks.return_value = [ network1, network2, network3 ]

        self.command.execute(progress=progress)

        self.assertEquals([
            '{0}{1}{2}'.format(network1['id'].ljust(36 + 5), "--".ljust(20), network1['name']),
            '{0}{1}{2}'.format(network2['id'].ljust(36 + 5), network2['cidr'].ljust(20), network2['name']),
            '{0}{1}{2}'.format(network3['id'].ljust(36 + 5), network3['cidr'].ljust(20), network3['name'])
        ], progress.getvalue().splitlines())



import sys

class Command(object):

    requires_api = False
    requires_deploy = False

    def __init__(self, rackspace_api=None, chef_deployer=None):
        self.rackspace_api = rackspace_api

    def execute(self, **kwargs):
        pass

    def validate_args(self, **kwargs):
        return True

class RackspaceCreate(Command):

    name = "create"
    description = "Create new Cloud Server and bootstrap Chef"
    requires_api = True
    requires_deploy = True

    def __init__(self, rackspace_api, chef_deployer):
        super(RackspaceCreate, self).__init__(rackspace_api)
        self.chef_deploy = chef_deployer

    def execute(self, node_name, flavor, image, public_key_file, environment=None, hostname=None, networks=None, **kwargs):
        host = self.rackspace_api.create_node(node_name=node_name, flavor=flavor,
                                              image=image, public_key_file=public_key_file,
                                              networks=networks,
                                              progress=sys.stderr)
        if environment:
            host.environment = environment
        if hostname:
            host.host_string = hostname

        self.chef_deploy.deploy(host=host, **kwargs)

    def validate_args(self, **kwargs):
        required_args = ["node_name", "flavor", "image"]
        for arg in required_args:
            if not kwargs.get(arg):
                return False

        return True

class RackspaceListImages(Command):

    name = "list-images"
    description = "List available images for a Cloud Servers endpoint"
    requires_api = True

    def execute(self, progress=sys.stderr, **kwargs):
        images = self.rackspace_api.list_images()
        for image in images:
            progress.write('{0}{1}\n'.format(image['id'].ljust(43), image['name']))

class RackspaceListFlavors(Command):

    name = "list-flavors"
    description = "List available flavors for a Cloud Servers endpoint"
    requires_api = True

    def execute(self, progress=sys.stderr, **kwargs):
        flavors = self.rackspace_api.list_flavors()
        for flavor in flavors:
            progress.write('{0}{1}\n'.format(flavor['id'].ljust(10), flavor['name']))

class RackspaceListNetworks(Command):

    name = "list-networks"
    description = "List available networks for a Cloud Servers endpoint"
    requires_api = True

    def execute(self, progress=sys.stderr, **kwargs):
        networks = self.rackspace_api.list_networks()

        for network in networks:
            cidr = network['cidr'] or '--'
            progress.write('{0}{1}{2}\n'.format(network['id'].ljust(41),
                                                cidr.ljust(20),
                                                network['name']))



from fabric.operations import sudo, os
import simplejson as json
from fabric.context_managers import hide
from littlechef import runner as lc
import littlechef

class ChefDeployer(object):

    def __init__(self, key_filename):
        self.key_filename = key_filename

    def deploy(self, host, runlist=None, plugins=None, post_plugins=None,
               use_opscode_chef=True, **kwargs):
        runlist = runlist or []
        plugins = plugins or []
        post_plugins = post_plugins or []

        self._setup_ssh_config(host)
        if use_opscode_chef:
            lc.deploy_chef(ask="no")

        self._save_node_data(host, runlist)
        for plugin in plugins:
            self._execute_plugin(host, plugin)

        self._bootstrap_node(host)

        for plugin in post_plugins:
            self._execute_plugin(host, plugin)

    def _save_node_data(self, host, runlist):
        """
        Save the runlist and environment into the node data
        """

        data = littlechef.lib.get_node(host.get_host_string())
        if host.environment:
            data['chef_environment'] = host.environment
        if runlist:
            data['run_list'] = runlist

        littlechef.chef.save_config(data, force=True)

    def _execute_plugin(self, host, plugin_name):
        node = littlechef.lib.get_node(host.get_host_string())
        plugin = littlechef.lib.import_plugin(plugin_name)
        littlechef.lib.print_header("Executing plugin '{0}' on "
                                    "{1}".format(plugin_name, lc.env.host_string))

        plugin.execute(node)

    def _setup_ssh_config(self, host):
        """
        New servers are created with 'root' user and an authorized public key
        that is a pair for the private key specified by self.key_filename.
        """

        # Set settings that don't have to do with our initial ssh config
        # (for example, encrypted data bag secret)
        littlechef.runner._readconfig()

        bootstrap_config_file = os.path.join(".", ".bootstrap-config_{0}".format(host.get_host_string()))
        contents = ("User root\n"
                    "IdentityFile {key_filename}\n"
                    "StrictHostKeyChecking no\n"
                    "Host {host_string}\n"
                    "HostName {ip_address}\n").format(key_filename=self.key_filename,
                                                      host_string=host.get_host_string(),
                                                      ip_address=host.ip_address)

        self._create_bootstrap_ssh_config(bootstrap_config_file, contents)

        # Use the ssh config we've created
        lc.env.use_ssh_config = True
        lc.env.ssh_config_path = bootstrap_config_file

        # Setup ssh config
        lc.env.user = "root"
        lc.env.host = host.get_host_string()
        lc.env.host_string = host.get_host_string()

    def _bootstrap_node(self, host):
        lc.node(host.get_host_string())

    def _create_bootstrap_ssh_config(self, bootstrap_config_file, contents):
        """
        Create temporary config file used for node bootstrapping
        """

        bootstrap_ssh_config = open(bootstrap_config_file, mode="w")
        bootstrap_ssh_config.write(contents)
        bootstrap_ssh_config.close()
        os.chmod(bootstrap_config_file, 0700)
        lc.env.ssh_config.parse(open(bootstrap_config_file))
